__title__ = 'twseia'
__description__ = """A Python library for Taiwan Smart Energy Industry Association (TaiSEIA) protocol."""
__url__ = 'https://twseia.readthedocs.io/'
__version__ = '0.0.1'
# __build__ = 0x022501
__author__ = 'Lee Shiueh'
__author_email__ = 'lee.shiueh@gmail.com'
__license__ = 'Apache 2.0'
# __copyright__ = 'Copyright 2021'
